<?php

namespace Services\Firebase;

use Services\Firebase\FCMServiceInterface;
use Plokko\Firebase\ServiceAccount;
use Plokko\Firebase\FCM\Request;
use Plokko\Firebase\FCM\Message;
use Plokko\Firebase\FCM\Targets\Token;
use Plokko\Firebase\FCM\Targets\Topic;
use Plokko\Firebase\FCM\Exceptions\FcmErrorException;

// TODO: refactor this implementation
class FCMService implements FCMServiceInterface{
    
    const DEFAULT_FILE_NAME = 'tecfood-mymenu-firebase-adminsdk.json';
    protected $defaultFilePath;

    public function __construct(){
        $this->defaultFilePath = realpath(implode(DIRECTORY_SEPARATOR, ['..', 'config', self::DEFAULT_FILE_NAME]));
    }

    function pathJoin(array $paths){
        return implode(DIRECTORY_SEPARATOR, $paths);
    }

    function getParcialPath(string $fullpath, string $path_limit, bool $include_path_limit = false){
        $position_subpath = strpos($fullpath, $path_limit);
        $include_path_limit = true;
        $result = [
            'MSG_ERROR'=> '',
            'SUB_PATH' => null
        ];
        try {
            if($position_subpath !== false) {
                $addPosition = $include_path_limit ? strlen($path_limit) : 0;
                $subpath = substr($fullpath, 0, $position_subpath+$addPosition);
                $real_sub_path = realpath($subpath);
                if($real_sub_path === false) {
                    $result['MSG_ERROR'] = 'INVALID_REAL_SUBPATH';
                } else {
                    $result['SUB_PATH'] = $real_sub_path;
                }
            } else {
                $result['MSG_ERROR'] = 'INVALID_PATH_LIMIT';
            }
        } catch (\Exception $e){
            $result['MSG_ERROR'] = 'UNEXPECTED_ERROR';
        }
        return $result;
    }

    private function getAuthFilePath(){
        $subPathToMatch = implode(DIRECTORY_SEPARATOR, ['mym', 'backend']);
        $moduleBasePath = $this->getParcialPath(__DIR__, $subPathToMatch, true)['SUB_PATH'];
        $fullPath = '';
        if( $moduleBasePath && false) {
            $fullPath = realpath($this->pathJoin([ $moduleBasePath, 'config', self::DEFAULT_FILE_NAME ]));
            if( $fullPath === false ) {
                throw new \Exception('INVALID_SUBPATH_AUTH_FIREBASE');
            }
        } else if($this->defaultFilePath) {
            $fullPath = $this->defaultFilePath;
        } else {
            throw new \Exception('INVALID_AUTH_FIREBASE');
        }
        return $fullPath;
    }
    
    public function sendPushNotificationToUser($userToken, $payload){
        $serviceAccount = new ServiceAccount($this->getAuthFilePath());
        $data = isset($payload['data']) ? $payload['data'] : null;
        $message = $this->getNewMessageFromToken($userToken, $payload['title'], $payload['body'], $data);
        $request = new Request($serviceAccount);
        try{
            $message->send($request);    
        } catch (FcmErrorException $fcmException){
            throw $fcmException;
        }
        
    }
    
    public function sendPushNotificationToTopic($topicName, $payload){
        $serviceAccount = new ServiceAccount($this->getAuthFilePath());
        $data = isset($payload['data']) ? $payload['data'] : null;
        $message = $this->getNewMessageFromTopicName($topicName, $payload['title'], $payload['body'], $data);
        $request = new Request($serviceAccount);
        try{
            $message->send($request);    
        } catch (FcmErrorException $fcmException){
            throw $fcmException;
        }
    }
    
    protected function getNewMessageFromToken($userToken, $title, $body, $data = null){
        $message = new Message();
        $message->notification->setTitle($title);
        $message->notification->setBody($body);
        $message->setTarget(new Token($userToken));
        if($data !== null){
            $message->data->fill($data);
        }
        return $message;
    }
    
    protected function getNewMessageFromTopicName($topicName, $title, $body, $data = null){
        $message = new Message();
        $message->notification->setTitle($title);
        $message->notification->setBody($body);
        $message->setTarget(new Topic($topicName));
        if($data !== null){
            $message->data->fill($data);
        }
        return $message;
    }
    
    
}