<?php
namespace Faturamento\Entities;

use Geral\Util\DateUtil;
use Setforms2\Shared\Registry;
use Geral\Util\Ambiente;

class Servclieturn extends Base\Servclieturn
{

    /**
     * Atualiza os turnos da Tela de CLientes por Serviço
     * @params $linhas
     * @params $cdcliente
     * @return bool.
     * @throws \Exception
     */
    public static function atualizaTurno($linhas, $cdcliente)
    {

        $connection = Registry::getEntityManager();
        $connection->beginTransaction();

        $nrorg = Ambiente::getNrorg();
        $data = DateUtil::getDataAtual();
        try {
            foreach ($linhas as $registro) {
                /**
                 * Traz os turnos do cliente filtrado
                 * @var \Faturamento\Entities\Servclieturn $servclieturn
                 */
                $servclieturn = Servclieturn::getObjectByWhere(
                    'CDFILIAL = ? AND NRORG = ? AND CDSERVFILI = ? AND CDTURNO = ? AND CDCLIENTE = ?',
                    array($registro['CDFILIAL'], $nrorg, $registro['CDSERVFILI'], $registro['CDTURNO'], $registro['CDCLIENTE'])
                );

                if (empty($servclieturn)) {

                    $servclieturn = new Servclieturn();
                    $registro['CDCLIENTE'] = $cdcliente;
                }

                $servclieturn->setCdfilial($registro['CDFILIAL']);
                $servclieturn->setCdservfili($registro['CDSERVFILI']);
                $servclieturn->setCdturno($registro['CDTURNO']);
                $servclieturn->setCdcliente($registro['CDCLIENTE']);
                $servclieturn->setVrservfattur($registro['VRSERVFATTUR']);
                $servclieturn->setDsemisturcli($registro['DSEMISTURCLI']);

                $servclieturn->setDtinclusao($data);
                $servclieturn->setDtultatu($data);
                $servclieturn->setNrorg($nrorg);
                $servclieturn->setNrorginclusao(Ambiente::getNrorg());
                $servclieturn->setCdoperinclusao(Ambiente::getCdOperador());
                $servclieturn->setNrorgultatu(Ambiente::getNrorg());
                $servclieturn->setCdoperultatu(Ambiente::getCdOperador());
                $servclieturn->persist();

            }
        } catch (\Exception $e) {
            throw $e;
        }
        $connection->flush();
        $connection->commit();
        return true;
    }

    /**
     * Atualiza as excessões por dia de cada Turno
     * @params $linhas
     * @return null
     * @throws \Exception
     */
    public static function atualizaExcessaoPorTurno($linhas)
    {
        try {

            $connection = Registry::getEntityManager();
            $connection->beginTransaction();
            foreach ($linhas as $registro) {

                $nrorg = Ambiente::getNrorg();
                $data = DateUtil::getDataAtual();


                /**
                 * Traz as execessões do turno escolhido
                 * @var \Faturamento\Entities\Servclieturn $servclieturn
                 */
                $servclieturn = Servclieturn::getObjectByWhere(
                    'CDFILIAL = ? AND NRORG = ? AND CDSERVFILI = ? AND CDTURNO = ? AND CDCLIENTE = ?',
                    array($registro['CDFILIAL'], $nrorg, $registro['CDSERVFILI'], $registro['CDTURNO'],
                        $registro['CDCLIENTE'])
                );


                $servclieturn->setCdfilial($registro['CDFILIAL']);
                $servclieturn->setCdservfili($registro['CDSERVFILI']);
                $servclieturn->setCdturno($registro['CDTURNO']);
                $servclieturn->setCdcliente($registro['CDCLIENTE']);

                switch ($registro['DIASEMANA']){
                case 'DOMINGO':
                        $servclieturn->setVrsvfattudom($registro['VRSERVFATTUR']);
                        $servclieturn->setDspremistudom($registro['DSEMISTURCLI']);
                    break;
                case 'SEGUNDA':
                        $servclieturn->setVrsvfattuseg($registro['VRSERVFATTUR']);
                        $servclieturn->setDspremistuseg($registro['DSEMISTURCLI']);
                    break;
                case 'TERÇA':
                        $servclieturn->setVrsvfattuter($registro['VRSERVFATTUR']);
                        $servclieturn->setDspremistuter($registro['DSEMISTURCLI']);
                    break;
                case 'QUARTA':
                        $servclieturn->setVrsvfattuqua($registro['VRSERVFATTUR']);
                        $servclieturn->setDspremistuqua($registro['DSEMISTURCLI']);
                    break;
                case 'QUINTA':
                        $servclieturn->setVrsvfattuqui($registro['VRSERVFATTUR']);
                        $servclieturn->setDspremistuqui($registro['DSEMISTURCLI']);
                    break;
                case 'SEXTA':
                        $servclieturn->setVrsvfattusex($registro['VRSERVFATTUR']);
                        $servclieturn->setDspremistusex($registro['DSEMISTURCLI']);
                    break;
                case 'SÁBADO':
                        $servclieturn->setVrsvfattusab($registro['VRSERVFATTUR']);
                        $servclieturn->setDspremistusab($registro['DSEMISTURCLI']);
                    break;
                case 'FERIADO':
                        $servclieturn->setVrsvfattufer($registro['VRSERVFATTUR']);
                        $servclieturn->setDspremistuter($registro['DSEMISTURCLI']);
                    break;
                }


                $servclieturn->setDtinclusao($data);
                $servclieturn->setDtultatu($data);
                $servclieturn->setNrorg($nrorg);
                $servclieturn->setNrorginclusao(Ambiente::getNrorg());
                $servclieturn->setCdoperinclusao(Ambiente::getCdOperador());
                $servclieturn->setNrorgultatu(Ambiente::getNrorg());
                $servclieturn->setCdoperultatu(Ambiente::getCdOperador());
                $servclieturn->persist();


            }
            $connection->flush();
            $connection->commit();

        } catch (\Exception $e) {
            throw $e;
        }
    }
}